from pydantic import BaseModel
from yacmmal.load.yaml import YAMLLoader
from yacmmal.load.json import JSONLoader
from yacmmal.load.toml import TOMLLoader
from yacmmal.types.formats import ConfigFormat
from yacmmal.types.config import ConfigAttrs
from typing import Sequence, Tuple, Callable, Type, Union

loaders = {
        ConfigFormat.YAML: YAMLLoader,
        ConfigFormat.JSON: JSONLoader,
        ConfigFormat.TOML: TOMLLoader
        }

def autoconfig(
    base_path: str,
    config: Sequence[Tuple[str, Union[str, ConfigAttrs], Type[BaseModel]]],
    format: Union[str, ConfigFormat],
    ) -> Callable:
    """
    Decorator to automatically load a config file.

    Parameters
    ----------
    base_path : str
        The base path to the config file.
    config : Sequence[Tuple[str, str, Type[BaseModel]]]
        A list of tuples containing the name of the config file, the path to the config file, and the dataclass.
    format : str
        The format of the config file.
    """
    if isinstance(format, str):
        format = ConfigFormat[format.upper()]

    def wrapper(func: Callable) -> Callable:
        def func_with_conf(*args, **kwargs):
            loader = loaders[format](base_path)
            for path, name, dclass in config:
                loader.add_path(path, name, dclass)
            return func(loader.extract(), *args, **kwargs)

        return func_with_conf

    return wrapper



import yaml
from typing import Type
from yacmmal.load.base import Loader
from pydantic import BaseModel

class YAMLLoader(Loader):
    """
    Loader for YAML files.

    Attributes
    ----------
    base_path : str
        The base path to the YAML files.
    """

    def __init__(self, base_path: str = "./"):
        super(YAMLLoader, self).__init__(base_path=base_path)
        self.format = "yaml"

    def load(self, path: str, dclass: Type[BaseModel]) -> BaseModel:
        """
        Loads a YAML file.

        Parameters
        ----------
        path : str
            The path to the YAML file.
        dclass : Type[BaseModel]
            The dataclass to use.

        Returns
        -------
        BaseModel
            The loaded model.
        """
        with open(path, "r") as f:
            data = yaml.load(f, Loader=yaml.Loader)
        return dclass.parse_obj(data)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yacmmal', 'yacmmal.load', 'yacmmal.types']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'pydantic>=1.9.0,<2.0.0', 'toml>=0.10.2,<0.11.0']

setup_kwargs = {
    'name': 'yacmmal',
    'version': '0.1.3',
    'description': 'Yet Another Config Manager for MAchine Learning (yacmmal) is a package to automatically load config files for machine learning projects.',
    'long_description': None,
    'author': 'Juan Lara',
    'author_email': 'julara@unal.edu.co',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
